==> fluent-mybatis-1.7.0-sources.jar: cn/org/atool/fluent/mybatis/mapper/MapperSql.java <==
package cn.org.atool.fluent.mybatis.mapper;


import cn.org.atool.fluent.mybatis.metadata.DbType;
import cn.org.atool.fluent.mybatis.segment.model.HintType;
import cn.org.atool.fluent.mybatis.segment.model.WrapperData;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import static cn.org.atool.fluent.mybatis.If.isBlank;
import static cn.org.atool.fluent.mybatis.If.notBlank;
import static cn.org.atool.fluent.mybatis.mapper.StrConstant.ASTERISK;
import static cn.org.atool.fluent.mybatis.mapper.StrConstant.SPACE;
import static java.util.stream.Collectors.joining;

/**
 * Mapper SQL组装
 *
 * @author darui.wu
 */
@SuppressWarnings({"unused", "UnusedReturnValue"})
public class MapperSql {
    private final StringBuilder buffer = new StringBuilder();

    @Override
    public String toString() {
        return buffer.toString().trim();
    }

    public MapperSql SELECT(String table, String columns) {
        buffer.append("SELECT ").append(columns).append(" FROM ").append(table);
        return this;
    }

    public MapperSql COUNT(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append("SELECT ");
        this.hint(data, HintType.After_CrudKey);
        buffer.append("COUNT(");
        String select = data.getSqlSelect();
        // select 单字段和多字段判断
        buffer.append(isBlank(select) || select.contains(",") ? ASTERISK : select.trim());
        buffer.append(") FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql INSERT_INTO(String table) {
        buffer.append("INSERT INTO ").append(table);
        return this;
    }

    public MapperSql VALUES() {
        buffer.append(" VALUES ");
        return this;
    }

    public MapperSql INSERT_COLUMNS(DbType dbType, List<String> columns) {
        String joining = columns.stream()
            .map(String::trim)
            .map(dbType::wrap)
            .collect(joining(", "));
        buffer.append("(").append(joining).append(")");
        return this;
    }

    public MapperSql INSERT_VALUES(List<String> values) {
        buffer.append("(").append(String.join(", ", values)).append(")");
        return this;
    }

    public MapperSql DELETE_FROM(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append(" DELETE ");
        this.hint(data, HintType.After_CrudKey);
        buffer.append(" FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql UPDATE(String table) {
        return this.UPDATE(table, null);
    }

    public MapperSql UPDATE(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append(" UPDATE ");
        this.hint(data, HintType.After_CrudKey);
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql SET(String... sets) {
        buffer.append(" SET ").append(String.join(",\n", sets));
        return this;
    }

    public MapperSql SET(List<String> sets) {
        buffer.append(" SET ").append(String.join(",\n", sets));
        return this;
    }

    public MapperSql WHERE(String whereSql) {
        buffer.append(" WHERE ").append(whereSql);
        return this;
    }

    public MapperSql WHERE(List<String> where) {
        buffer.append(" WHERE ").append(String.join(" AND ", where));
        return this;
    }

    public MapperSql WHERE(DbType dbType, String prefix, Map<String, Object> where) {
        List<String> ands = new ArrayList<>();
        for (Map.Entry<String, Object> entry : where.entrySet()) {
            if (entry.getValue() == null) {
                ands.add(dbType.wrap(entry.getKey()) + " IS NULL");
            } else {
                String column = entry.getKey();
                String el = dbType.wrap(column) + " = " + "#{" + (isBlank(prefix) ? column : prefix + "." + column) + "}";
                ands.add(el);
            }
        }
        return this.WHERE(ands);
    }

    public MapperSql WHERE_GROUP_BY(WrapperData data) {
        if (notBlank(data.getWhereSql())) {
            this.WHERE(data.getWhereSql());
        }
        if (notBlank(data.getGroupBy())) {
            this.APPEND(data.getGroupBy());
        }
        if (notBlank(data.getLastSql())) {
            this.APPEND(data.getLastSql());
        }
        return this;
    }

    public MapperSql WHERE_GROUP_ORDER_BY(WrapperData data) {
        if (notBlank(data.getWhereSql())) {
            this.WHERE(data.getWhereSql());
        }
        if (notBlank(data.getGroupBy())) {
            this.APPEND(data.getGroupBy());
        }
        if (notBlank(data.getOrderBy())) {
            this.APPEND(data.getOrderBy());
        }
        if (notBlank(data.getLastSql())) {
            this.APPEND(data.getLastSql());
        }
        return this;
    }

    public MapperSql WHERE_PK_IN(String column, int size) {
        buffer.append(" WHERE ").append(column).append(" IN (");
        for (int index = 0; index < size; index++) {
            if (index > 0) {
                buffer.append(", ");
            }
            buffer.append("#{list[").append(index).append("]}");
        }
        buffer.append(")");
        return this;
    }

    public MapperSql APPEND(String sql) {
        buffer.append(SPACE).append(sql).append(SPACE);
        return this;
    }

    public MapperSql SELECT(String table, WrapperData data, String defaultColumns) {
        this.hint(data, HintType.Before_All);
        buffer.append("SELECT ");
        this.hint(data, HintType.After_CrudKey);
        this.APPEND(data.isDistinct() ? "DISTINCT " : SPACE);
        buffer.append(isBlank(data.getSqlSelect()) ? defaultColumns : data.getSqlSelect());
        buffer.append(" FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    /**
     * 添加limit语句
     *
     * @param data           WrapperData
     * @param offsetEverZero 永远从0开始情况
     * @return MapperSql
     */
    public MapperSql LIMIT(WrapperData data, boolean offsetEverZero) {
        if (data == null || data.getPaged() == null) {
            return this;
        }
        if (offsetEverZero) {
            this.APPEND(" LIMIT #{ew.wrapperData.paged.limit}");
        } else {
            this.APPEND(" LIMIT #{ew.wrapperData.paged.offset}, #{ew.wrapperData.paged.limit}");
        }
        return this;
    }

    private void hint(WrapperData data, HintType hintType) {
        if (data != null) {
            buffer.append(data.hint(hintType));
        }
    }
}

==> fluent-mybatis-1.8.1-sources.jar: cn/org/atool/fluent/mybatis/mapper/MapperSql.java <==
package cn.org.atool.fluent.mybatis.mapper;


import cn.org.atool.fluent.mybatis.base.crud.IQuery;
import cn.org.atool.fluent.mybatis.metadata.DbType;
import cn.org.atool.fluent.mybatis.segment.model.HintType;
import cn.org.atool.fluent.mybatis.segment.model.WrapperData;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicLong;

import static cn.org.atool.fluent.mybatis.If.isBlank;
import static cn.org.atool.fluent.mybatis.If.notBlank;
import static cn.org.atool.fluent.mybatis.mapper.StrConstant.ASTERISK;
import static cn.org.atool.fluent.mybatis.mapper.StrConstant.SPACE;
import static java.util.stream.Collectors.joining;

/**
 * Mapper SQL组装
 *
 * @author darui.wu
 */
@SuppressWarnings({"unused", "UnusedReturnValue"})
public class MapperSql {
    private final StringBuilder buffer = new StringBuilder();

    @Override
    public String toString() {
        return buffer.toString().trim();
    }

    public MapperSql SELECT(String table, String columns) {
        buffer.append("SELECT ").append(columns).append(" FROM ").append(table);
        return this;
    }

    public MapperSql COUNT(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append("SELECT ");
        this.hint(data, HintType.After_CrudKey);
        buffer.append("COUNT(");
        String select = data.getSqlSelect();
        // select 单字段和多字段判断
        buffer.append(isBlank(select) || select.contains(",") ? ASTERISK : select.trim());
        buffer.append(") FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql INSERT_INTO(String table) {
        buffer.append("INSERT INTO ").append(table);
        return this;
    }

    public MapperSql VALUES() {
        buffer.append(" VALUES ");
        return this;
    }

    public MapperSql INSERT_COLUMNS(DbType dbType, List<String> columns) {
        String joining = columns.stream()
            .map(String::trim)
            .map(dbType::wrap)
            .collect(joining(", "));
        buffer.append(brackets(joining));
        return this;
    }

    public MapperSql INSERT_VALUES(List<String> values) {
        buffer.append(brackets(String.join(", ", values)));
        return this;
    }

    public MapperSql DELETE_FROM(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append(" DELETE ");
        this.hint(data, HintType.After_CrudKey);
        buffer.append(" FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql UPDATE(String table) {
        return this.UPDATE(table, null);
    }

    public MapperSql UPDATE(String table, WrapperData data) {
        this.hint(data, HintType.Before_All);
        buffer.append(" UPDATE ");
        this.hint(data, HintType.After_CrudKey);
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    public MapperSql SET(String... sets) {
        buffer.append(" SET ").append(String.join(",\n", sets));
        return this;
    }

    public MapperSql SET(List<String> sets) {
        buffer.append(" SET ").append(String.join(",\n", sets));
        return this;
    }

    public MapperSql WHERE(String whereSql) {
        buffer.append(" WHERE ").append(whereSql);
        return this;
    }

    public MapperSql WHERE(List<String> where) {
        buffer.append(" WHERE ").append(String.join(" AND ", where));
        return this;
    }

    public MapperSql WHERE(DbType dbType, String prefix, Map<String, Object> where) {
        List<String> ands = new ArrayList<>();
        for (Map.Entry<String, Object> entry : where.entrySet()) {
            if (entry.getValue() == null) {
                ands.add(dbType.wrap(entry.getKey()) + " IS NULL");
            } else {
                String column = entry.getKey();
                String el = dbType.wrap(column) + " = " + "#{" + (isBlank(prefix) ? column : prefix + "." + column) + "}";
                ands.add(el);
            }
        }
        return this.WHERE(ands);
    }

    public MapperSql WHERE_GROUP_BY(WrapperData data) {
        if (notBlank(data.getWhereSql())) {
            this.WHERE(data.getWhereSql());
        }
        if (notBlank(data.getGroupBy())) {
            this.APPEND(data.getGroupBy());
        }
        if (notBlank(data.getLastSql())) {
            this.APPEND(data.getLastSql());
        }
        return this;
    }

    public MapperSql WHERE_GROUP_ORDER_BY(WrapperData data) {
        if (notBlank(data.getWhereSql())) {
            this.WHERE(data.getWhereSql());
        }
        if (notBlank(data.getGroupBy())) {
            this.APPEND(data.getGroupBy());
        }
        if (notBlank(data.getOrderBy())) {
            this.APPEND(data.getOrderBy());
        }
        if (notBlank(data.getLastSql())) {
            this.APPEND(data.getLastSql());
        }
        return this;
    }

    public MapperSql APPEND(String sql) {
        buffer.append(SPACE).append(sql).append(SPACE);
        return this;
    }

    public MapperSql SELECT(String table, WrapperData data, String defaultColumns) {
        this.hint(data, HintType.Before_All);
        buffer.append("SELECT ");
        this.hint(data, HintType.After_CrudKey);
        this.APPEND(data.isDistinct() ? "DISTINCT " : SPACE);
        buffer.append(isBlank(data.getSqlSelect()) ? defaultColumns : data.getSqlSelect());
        buffer.append(" FROM ");
        this.hint(data, HintType.Before_Table);
        buffer.append(table);
        this.hint(data, HintType.After_Table);
        return this;
    }

    /**
     * 添加limit语句
     *
     * @param data           WrapperData
     * @param offsetEverZero 永远从0开始情况
     * @return MapperSql
     */
    public MapperSql LIMIT(WrapperData data, boolean offsetEverZero) {
        if (data == null || data.getPaged() == null) {
            return this;
        }
        if (offsetEverZero) {
            this.APPEND(" LIMIT #{ew.wrapperData.paged.limit}");
        } else {
            this.APPEND(" LIMIT #{ew.wrapperData.paged.offset}, #{ew.wrapperData.paged.limit}");
        }
        return this;
    }

    private void hint(WrapperData data, HintType hintType) {
        if (data != null) {
            buffer.append(data.hint(hintType));
        }
    }

    /**
     * 给 query sql 加上括弧
     *
     * @param query IQuery
     * @return (query sql)
     */
    public static String brackets(IQuery query) {
        return brackets(query.getWrapperData().sqlWithoutPaged());
    }

    /**
     * 给sql加上括弧
     *
     * @param sql sql
     * @return (sql)
     */
    public static String brackets(MapperSql sql) {
        return brackets(sql.toString());
    }

    /**
     * 给sql加上括弧
     *
     * @param sql sql
     * @return (sql)
     */
    public static String brackets(String sql) {
        if (sql == null) {
            return null;
        } else if (sql.startsWith("(") && sql.endsWith(")")) {
            return sql;
        } else {
            return "(" + sql.trim() + ")";
        }
    }

    static final AtomicLong tmp = new AtomicLong(0);

    /**
     * 临时表别名
     *
     * @return 临时表别名
     */
    public static String tmpTable() {
        return "TMP_" + tmp.incrementAndGet();
    }
}
